@@ -24,14 +24,19 @@ package Spreadsheet::Read;
 
  # OO API
  my $book = Spreadsheet::Read->new ("file.csv");
+ my $sheet = $book->sheet (1);
+ my $cell  = $sheet->cell ("A3");
+ my $cell  = $sheet->cell (1, 3);
+
  $book->add ("test.xls");
 
 =cut
 
+use 5.8.1;
 use strict;
 use warnings;
 
-our $VERSION = "0.69";
+our $VERSION = "0.78";
 sub  Version { $VERSION }
 
 use Carp;
@@ -60,42 +65,44 @@ my @parsers = (
     [ dmp  => "Data::Peek",				""		],
     );
 my %can = map {
-    my $preset = $ENV{"SPREADSHEET_READ_\U$_->[0]"};
-    if ($preset) {
-	eval "require $preset";
-	if ($@) {
-	    $preset = "!$preset";
-	    }
-	else { # forcing a parser should still check the version
-	    for (grep { $_->[1] eq $preset and $_->[2] } @parsers) {
+    my $p = $_;
+    my $preset = $ENV{"SPREADSHEET_READ_\U$p->[0]"};
+    if ($preset and $preset =~ m/^[\w:]+$/) {
+	if (eval "require $preset" and not $@) {
+	    # forcing a parser should still check the version
+	    for (grep { $p->[1] eq $preset and $p->[2] } @parsers) {
 		my $ok;
-		if ($_->[2] =~ m/^v([0-9.]+)/) {	# clumsy versions
+		my $has = $preset->VERSION;
+		$has =~ s/_[0-9]+$//;			# Remove beta-part
+		if ($p->[2] =~ m/^v([0-9.]+)/) {	# clumsy versions
 		    my @min = split m/\./ => $1;
-		    my $has = $preset->VERSION;
 		    $has =~ s/^v//;
 		    my @has = split m/\./ => $has;
 		    $ok = (($has[0] * 1000 + $has[1]) * 1000 + $has[2]) >=
 			  (($min[0] * 1000 + $min[1]) * 1000 + $min[2]);
 		    }
 		else {	# normal versions
-		    $ok = $preset->VERSION >= $_->[2];
+		    $ok = $has >= $p->[2];
 		    }
 		$ok or $preset = "!$preset";
 		}
 	    }
+	else {
+	    $preset = "!$preset";
+	    }
 	}
-    $_->[0] => $preset || "";
+    $p->[0] => $preset || "";
     } @parsers;
 for (@parsers) {
     my ($flag, $mod, $vsn) = @$_;
     $can{$flag} and next;
-    eval "require $mod; \$vsn and ${mod}->VERSION (\$vsn); \$can{\$flag} = '$mod'";
-    #$@ && $@ !~ m{Can't locate} and warn "$mod: $@\n";
+    eval "require $mod; \$vsn and ${mod}->VERSION (\$vsn); \$can{\$flag} = '$mod'" or
+	$_->[0] = "! Cannot use $mod version $vsn: $@";
     }
 $can{sc} = __PACKAGE__;	# SquirelCalc is built-in
 
-$can{xlsx} =~ m/LibXML/ && $] < 5.012 and
-    substr $can{xlsx}, 0, 0, "!"; # This parser requires perl 5.12 or newer
+defined $Spreadsheet::ParseExcel::VERSION && $Spreadsheet::ParseExcel::VERSION < 0.61 and
+    *Spreadsheet::ParseExcel::Workbook::get_active_sheet = sub { undef; };
 
 my $debug = 0;
 my %def_opts = (
@@ -104,11 +111,14 @@ my %def_opts = (
     attr    => 0,
     clip    => undef, # $opt{cells};
     strip   => 0,
+    pivot   => 0,
     dtfmt   => "yyyy-mm-dd", # Format 14
     debug   => 0,
+    passwd  => undef,
     parser  => undef,
     sep     => undef,
     quote   => undef,
+    label   => undef,
     );
 my @def_attr = (
     type    => "text",
@@ -182,12 +192,11 @@ sub sheets {
     wantarray ? sort { $s{$a} <=> $s{$b} } keys %s : $ctrl->{sheets};
     } # sheets
 
-# cr2cell (4, 18) => "D18"
-# No prototype to allow 'cr2cell (@rowcol)'
-sub cr2cell {
+# col2label (4) => "D"
+sub col2label {
     ref $_[0] eq __PACKAGE__ and shift;
-    my ($c, $r) = @_;
-    defined $c && defined $r && $c > 0 && $r > 0 or return "";
+    my $c = shift;
+    defined $c && $c > 0 or return "";
     my $cell = "";
     while ($c) {
 	use integer;
@@ -195,7 +204,16 @@ sub cr2cell {
 	substr $cell, 0, 0, chr (--$c % 26 + ord "A");
 	$c /= 26;
 	}
-    "$cell$r";
+    $cell;
+    } # col2label
+
+# cr2cell (4, 18) => "D18"
+# No prototype to allow 'cr2cell (@rowcol)'
+sub cr2cell {
+    ref $_[0] eq __PACKAGE__ and shift;
+    my ($c, $r) = @_;
+    defined $c && defined $r && $c > 0 && $r > 0 or return "";
+    col2label ($c) . $r;
     } # cr2cell
 
 # cell2cr ("D18") => (4, 18)
@@ -253,6 +271,10 @@ sub sheet {
 	return bless $book->[$sheet]			=> $class;
     exists $book->[0]{sheet}{$sheet} and
 	return bless $book->[$book->[0]{sheet}{$sheet}]	=> $class;
+    foreach my $idx (1 .. $book->[0]{sheets}) {
+	$book->[$idx]{label} eq $sheet and
+	    return bless $book->[$idx]			=> $class;
+	}
     return;
     } # sheet
 
@@ -261,51 +283,85 @@ sub sheet {
 sub _clipsheets {
     my ($opt, $ref) = @_;
 
-    if (my $s = $opt->{strip} and $ref->[0]{sheets}) {
+    $ref->[0]{sheets} or return $ref;
+
+    my ($rc, $cl)      = ($opt->{rc},   $opt->{cells});
+    my ($oc, $os, $oa) = ($opt->{clip}, $opt->{strip}, $opt->{attr});
+
+    # Strip leading/trailing spaces
+    if ($os || $oc) {
 	foreach my $sheet (1 .. $ref->[0]{sheets}) {
+	    $ref->[$sheet]{indx} = $sheet;
 	    my $ss = $ref->[$sheet];
 	    $ss->{maxrow} && $ss->{maxcol} or next;
+	    my ($mc, $mr) = (0, 0);
 	    foreach my $row (1 .. $ss->{maxrow}) {
 		foreach my $col (1 .. $ss->{maxcol}) {
-		    for (($opt->{rc}    ? $ss->{cell}   [$col][$row]  : ()),
-		         ($opt->{cells} ? $ss->{cr2cell ($col, $row)} : ())) {
-			defined or next;
-		        $s & 2 and s/\s+$//;
-			$s & 1 and s/^\s+//;
+		    if ($rc) {
+			defined $ss->{cell}[$col][$row] or next;
+			$os & 2 and $ss->{cell}[$col][$row] =~ s/\s+$//;
+			$os & 1 and $ss->{cell}[$col][$row] =~ s/^\s+//;
+			if (length $ss->{cell}[$col][$row]) {
+			    $col > $mc and $mc = $col;
+			    $row > $mr and $mr = $row;
+			    }
+			}
+		    if ($cl) {
+			my $cell = cr2cell ($col, $row);
+			defined $ss->{$cell} or next;
+			$os & 2 and $ss->{$cell} =~ s/\s+$//;
+			$os & 1 and $ss->{$cell} =~ s/^\s+//;
+			if (length $ss->{$cell}) {
+			    $col > $mc and $mc = $col;
+			    $row > $mr and $mr = $row;
+			    }
 			}
 		    }
 		}
+
+	    $oc && ($mc < $ss->{maxcol} || $mr < $ss->{maxrow}) or next;
+
+	    # Remove trailing empty columns
+	    foreach my $col (($mc + 1) .. $ss->{maxcol}) {
+		$rc and undef $ss->{cell}[$col];
+		$oa and undef $ss->{attr}[$col];
+		$cl or next;
+		my $c = col2label ($col);
+		delete $ss->{"$c$_"} for 1 .. $ss->{maxrow};
+		}
+
+	    # Remove trailing empty rows
+	    foreach my $row (($mr + 1) .. $ss->{maxrow}) {
+		foreach my $col (1 .. $mc) {
+		    $cl and delete $ss->{cr2cell ($col, $row)};
+		    $rc and undef  $ss->{cell}   [$col][$row];
+		    $oa and undef  $ss->{attr}   [$col][$row];
+		    }
+		}
+
+	    ($ss->{maxrow}, $ss->{maxcol}) = ($mr, $mc);
 	    }
 	}
 
-    $opt->{clip} or return $ref;
-
-    foreach my $sheet (1 .. $ref->[0]{sheets}) {
-	my $ss = $ref->[$sheet];
-
-	# Remove trailing empty columns
-	while ($ss->{maxcol} and not (
-		grep { defined && m/\S/ } @{$ss->{cell}[$ss->{maxcol}]})
-		) {
-	    (my $col = cr2cell ($ss->{maxcol}, 1)) =~ s/1$//;
-	    my $recol = qr{^$col(?=[0-9]+)$};
-	    delete $ss->{$_} for grep m/$recol/, keys %{$ss};
-	    $ss->{maxcol}--;
+    if ($opt->{pivot}) {
+	foreach my $sheet (1 .. $ref->[0]{sheets}) {
+	    my $ss = $ref->[$sheet];
+	    $ss->{maxrow} || $ss->{maxcol} or next;
+	    my $mx = $ss->{maxrow} > $ss->{maxcol} ? $ss->{maxrow} : $ss->{maxcol};
+	    foreach my $row (2 .. $mx) {
+		foreach my $col (1 .. ($row - 1)) {
+		    $opt->{rc}    and
+			($ss->{cell}[$col][$row], $ss->{cell}[$row][$col]) =
+			($ss->{cell}[$row][$col], $ss->{cell}[$col][$row]);
+		    $opt->{cells} and
+		        ($ss->{cr2cell ($col, $row)}, $ss->{cr2cell ($row, $col)}) =
+		        ($ss->{cr2cell ($row, $col)}, $ss->{cr2cell ($col, $row)});
+		    }
+		}
+	    ($ss->{maxcol}, $ss->{maxrow}) = ($ss->{maxrow}, $ss->{maxcol});
 	    }
-	$ss->{maxcol} or $ss->{maxrow} = 0;
-
-	# Remove trailing empty rows
-	while ($ss->{maxrow} and not (
-		grep { defined && m/\S/ }
-		map  { $ss->{cell}[$_][$ss->{maxrow}] }
-		1 .. $ss->{maxcol}
-		)) {
-	    my $rerow = qr{^[A-Z]+$ss->{maxrow}$};
-	    delete $ss->{$_} for grep m/$rerow/, keys %{$ss};
-	    $ss->{maxrow}--;
-	    }
-	$ss->{maxrow} or $ss->{maxcol} = 0;
 	}
+
     $ref;
     } # _clipsheets
 
@@ -330,88 +386,6 @@ sub _xls_fill {
     return _xls_color ($bg);
     } # _xls_fill
 
-sub _xlsx_libxml {
-    my $oBook = shift;
-    my @names = @{$oBook->get_worksheet_names};
-    $oBook->{SheetCount} = scalar @names;
-    $oBook->{Worksheet}  = [ ];
-    my %mm;
-    while (my $wks = $oBook->worksheet) {
-	push @{$oBook->{Worksheet}}, $wks;
-	$wks->{Cells}  = [];
-	$wks->{Name}   = shift @names;
-	($wks->{MinRow}, $wks->{MaxRow}) = $wks->row_range;
-	($wks->{MinCol}, $wks->{MaxCol}) = $wks->col_range;
-	$wks->{MinRow} < 0 and $wks->{MinRow} = 0;
-	$wks->{MinCol} < 0 and $wks->{MinCol} = 0;
-	foreach my $r ($wks->{MinRow} .. $wks->{MaxRow}) {
-	    foreach my $c ($wks->{MinCol} .. $wks->{MaxCol}) {
-		my $cell = $wks->get_cell ($r, $c);
-		if (defined $cell and ref $cell) {
-		    $cell->{Val}    = $cell->unformatted;
-		    $cell->{Merged} = $cell->is_merged and
-			$mm{$cell->{cell_merge}}{"$r:$c"} = $cell;
-		    $cell->{Type}   = $cell->type;
-		    $cell->{Hidden} = 0;#$cell->is_hidden; NYI
-		    }
-		else {
-		    $cell = {
-			Val    => undef,
-			Type   => "Undef",
-			Merged => 0,
-			Hidden => 0,
-			};
-		    }
-		$wks->{Cells}[$r][$c] = $cell;
-		}
-	    }
-	# Spreadsheet::XLSX::Reader::LibXML returned all other cells in range as undef
-	foreach my $mm (values %mm) {
-	    my @mr = sort keys %$mm;
-	    foreach my $rc (@mr) {
-		defined $mm->{$rc}{Val} or $mm->{$rc}{Val} = "";
-		}
-	    }
-	}
-    *WorksheetInstance::get_merged_areas = sub {
-	my $wi = shift or return;
-	my $mm = eval { $wi->_get_merge_map } ||
-		 eval { $wi->_merge_map     } or return;
-	# [ undef,
-	#   [ undef,
-	#     undef,
-	#     'B1:C2',
-	#     'B1:C2'
-	#     ],
-	#   [ undef,
-	#     'A2:A3',
-	#     'B1:C2',
-	#     'B1:C2'
-	#     ],
-	#   [ undef,
-	#     'A2:A3'
-	#     ]
-	#   ]
-	# ->
-	# [ [ 1, 0,	# A2:
-	#     2, 0,	#  A3
-	#     ],
-	#   [ 0, 1,	# B1:
-	#     1, 2,	#  C2
-	#     ]
-	#   ]
-	my %r;
-	for (@$mm) { $_ && $r{$_}++ for @$_ }
-	keys %r or return;
-	my @r;
-	foreach my $ma (keys %r) {
-	    my ($ul, $br) = split m/:/ => $ma or return;
-	    push @r, [ reverse map { $_ - 1 } map { cell2cr ($_) } $br, $ul ];
-	    }
-	return \@r;
-	};
-    } # _xlsx_libxml
-
 sub ReadData {
     my $txt = shift	or  return;
 
@@ -436,18 +410,20 @@ sub ReadData {
 		      grep { !exists $def_opts{$_} }
 		      keys %opt;
 
+    my $_parser = _parser ($opt{parser});
+
     my $io_ref = ref ($txt) =~ m/GLOB|IO/ ? $txt : undef;
-    my $io_fil = $io_ref ? 0 : do { no warnings "newline"; -f $txt ? 1 : 0 };
+    my $io_fil = $io_ref ? 0 : $txt =~ m/\0/ ? 0
+			     : do { no warnings "newline"; -f $txt };
     my $io_txt = $io_ref || $io_fil ? 0 : 1;
 
     $io_fil && ! -s $txt  and return;
     $io_ref && eof ($txt) and return;
 
-    if ($opt{parser} ? _parser ($opt{parser}) eq "csv"
-		     : ($io_fil && $txt =~ m/\.(csv)$/i)) {
+    if ($opt{parser} ? $_parser eq "csv" : ($io_fil && $txt =~ m/\.(csv)$/i)) {
 	$can{csv} or croak "CSV parser not installed";
 
-	my $label = $io_fil ? $txt : "IO";
+	my $label = defined $opt{label} ? $opt{label} : $io_fil ? $txt : "IO";
 
 	$debug and print STDERR "Opening CSV $label using $can{csv}-", $can{csv}->VERSION, "\n";
 
@@ -473,34 +449,48 @@ sub ReadData {
 		cell	=> [],
 		attr	=> [],
 		merged	=> [],
+		active  => 1,
 		},
 	    );
 
 	my ($sep, $quo, $in) = (",", '"');
 	defined $opt{sep}   and $sep = $opt{sep};
 	defined $opt{quote} and $quo = $opt{quote};
+	$debug > 8 and _dump (debug => {
+	    data => \@data, txt => $txt, io_ref => $io_ref, io_fil => $io_fil });
 	if ($io_fil) {
 	    unless (defined $opt{quote} && defined $opt{sep}) {
 		open $in, "<", $txt or return;
-		$_ = <$in>;
+		my $l1 = <$in>;
 
 		$quo = defined $opt{quote} ? $opt{quote} : '"';
 		$sep = # If explicitly set, use it
 		   defined $opt{sep} ? $opt{sep} :
 		       # otherwise start auto-detect with quoted strings
-		       m/["0-9];["0-9;]/	? ";"  :
-		       m/["0-9],["0-9,]/	? ","  :
-		       m/["0-9]\t["0-9,]/	? "\t" :
+		       $l1 =~ m/["0-9];["0-9;]/  ? ";"  :
+		       $l1 =~ m/["0-9],["0-9,]/  ? ","  :
+		       $l1 =~ m/["0-9]\t["0-9,]/ ? "\t" :
 		       # If neither, then for unquoted strings
-		       m/\w;[\w;]/		? ";"  :
-		       m/\w,[\w,]/		? ","  :
-		       m/\w\t[\w,]/		? "\t" :
-					      ","  ;
+		       $l1 =~ m/\w;[\w;]/        ? ";"  :
+		       $l1 =~ m/\w,[\w,]/        ? ","  :
+		       $l1 =~ m/\w\t[\w,]/       ? "\t" :
+						   ","  ;
 		close $in;
 		}
 	    open $in, "<", $txt or return;
 	    }
+	elsif ($io_ref) {
+	    $in = $txt;
+	    }
+	elsif (ref $txt eq "SCALAR") {
+	    open $in, "<", $txt  or croak "Cannot open input: $!";
+	    }
+	elsif ($txt =~ m/[\r\n,;]/) {
+	    open $in, "<", \$txt or croak "Cannot open input: $!";
+	    }
 	else {
+	    warn "Input type ", ref $txt,
+		" might not be supported. Please file a ticket\n";
 	    $in = $txt;	# Now pray ...
 	    }
 	$debug > 1 and print STDERR "CSV sep_char '$sep', quote_char '$quo'\n";
@@ -537,7 +527,7 @@ sub ReadData {
 	}
 
     # From /etc/magic: Microsoft Office Document
-    if ($io_txt && _parser ($opt{parser}) !~ m/^xlsx?$/ &&
+    if ($io_txt && $_parser !~ m/^xlsx?$/ &&
 		    $txt =~ m{^(\376\067\0\043
 			       |\320\317\021\340\241\261\032\341
 			       |\333\245-\0\0\0)}x) {
@@ -554,16 +544,15 @@ sub ReadData {
 	    }
 	open $io_ref, "<", $tmpfile or return;
 	$io_txt = 0;
-	$opt{parser} = "xls";
+	$_parser = _parser ($opt{parser} = "xls");
 	}
-    my $_parser;
-    if ($opt{parser} ? ($_parser = _parser ($opt{parser})) =~ m/^xlsx?$/
+    if ($opt{parser} ? $_parser =~ m/^xlsx?$/
 		     : ($io_fil && $txt =~ m/\.(xlsx?)$/i && ($_parser = $1))) {
 	my $parse_type = $_parser =~ m/x$/i ? "XLSX" : "XLS";
 	my $parser = $can{lc $parse_type} or
 	    croak "Parser for $parse_type is not installed";
-	my $xlsx_libxml = $parser =~ m/LibXML$/;
 	$debug and print STDERR "Opening $parse_type $txt using $parser-", $can{lc $parse_type}->VERSION, "\n";
+	$opt{passwd} and $parser_opts{Password} = $opt{passwd};
 	my $oBook = eval {
 	    $io_ref
 	      ? $parse_type eq "XLSX"
@@ -592,8 +581,6 @@ sub ReadData {
 	# _buffer        FmtClass       PkgStr         Worksheet
 	# CellHandler    Font           _previous_info
 
-	$xlsx_libxml and _xlsx_libxml ($oBook);
-
 	my @data = ( {
 	    type	=> lc $parse_type,
 	    parser	=> $can{lc $parse_type},
@@ -623,7 +610,12 @@ sub ReadData {
 	    :     Spreadsheet::ParseExcel::FmtDefault->new;
 
 	$debug and print STDERR "\t$data[0]{sheets} sheets\n";
+	my $active_sheet = $oBook->get_active_sheet
+			|| $oBook->{ActiveSheet}
+			|| $oBook->{SelectedSheet};
+	my $current_sheet = 0;
 	foreach my $oWkS (@{$oBook->{Worksheet}}) {
+	    $current_sheet++;
 	    $opt{clip} and !defined $oWkS->{Cells} and next; # Skip empty sheets
 	    my %sheet = (
 		parser	=> 0,
@@ -633,6 +625,7 @@ sub ReadData {
 		cell	=> [],
 		attr	=> [],
 		merged  => [],
+		active	=> 0,
 		);
 	    # $debug and $sheet{_parser} = $oWkS;
 	    defined $sheet{label}  or  $sheet{label}  = "-- unlabeled --";
@@ -648,6 +641,11 @@ sub ReadData {
 		@{$oWkS->get_merged_areas || []}];
 	    my $sheet_idx = 1 + @data;
 	    $debug and print STDERR "\tSheet $sheet_idx '$sheet{label}' $sheet{maxrow} x $sheet{maxcol}\n";
+	    if (defined $active_sheet) {
+		# _SheetNo is 0-based
+		my $sheet_no = defined $oWkS->{_SheetNo} ? $oWkS->{_SheetNo} : $current_sheet - 1;
+		$sheet_no eq $active_sheet and $sheet{active} = 1;
+		}
 	    # Sheet keys:
 	    # _Book          FooterMargin   MinCol         RightMargin
 	    # BottomMargin   FooterMergin   MinRow         RightMergin
@@ -800,6 +798,7 @@ sub ReadData {
 	    close   $sc;
 	    }
 	$txt =~ m/\S/ or return;
+	my $label = defined $opt{label} ? $opt{label} : "sheet";
 	my @data = (
 	    {	type	=> "sc",
 		parser	=> "Spreadsheet::Read",
@@ -811,15 +810,16 @@ sub ReadData {
 		    }],
 		error	=> undef,
 		sheets	=> 1,
-		sheet	=> { sheet => 1 },
+		sheet	=> { $label => 1 },
 		},
 	    {	parser	=> 0,
-		label	=> "sheet",
+		label	=> $label,
 		maxrow	=> 0,
 		maxcol	=> 0,
 		cell	=> [],
 		attr	=> [],
 		merged  => [],
+		active  => 1,
 		},
 	    );
 
@@ -849,6 +849,9 @@ sub ReadData {
     if ($opt{parser} ? _parser ($opt{parser}) eq "sxc"
 		     : ($txt =~ m/^<\?xml/ or -f $txt)) {
 	$can{sxc} or croak "Spreadsheet::ReadSXC not installed";
+	ref $txt and
+	    croak ("Sorry, references as input are not (yet) supported by Spreadsheet::ReadSXC");
+
 	my $using = "using $can{sxc}-" . $can{sxc}->VERSION;
 	my $sxc_options = { %parser_opts, OrderBySheet => 1 }; # New interface 0.20 and up
 	my $sxc;
@@ -903,6 +906,7 @@ sub ReadData {
 		    cell   => [],
 		    attr   => [],
 		    merged => [],
+		    active => 0,
 		    );
 		my $sheet_idx = 1 + @data;
 		$debug and print STDERR "\tSheet $sheet_idx '$sheet{label}' $sheet{maxrow} rows\n";
@@ -936,7 +940,8 @@ sub ReadData {
 sub add {
     my $book = shift;
     my $r = ReadData (@_) or return;
-    $book && (ref $book eq "ARRAY" || ref $book eq __PACKAGE__) && $book->[0]{sheets} or return $r;
+    $book && (ref $book eq "ARRAY" ||
+	      ref $book eq __PACKAGE__) && $book->[0]{sheets} or return $r;
 
     my $c1 = $book->[0];
     my $c2 = $r->[0];
@@ -979,6 +984,17 @@ sub cell {
 	return $sheet->{$id[0]};
     } # cell
 
+sub attr {
+    my ($sheet, @id) = @_;
+    @id == 2 && $id[0] =~ m/^[0-9]+$/ && $id[1] =~ m/^[0-9]+$/ and
+	return $sheet->{attr}[$id[0]][$id[1]];
+    if (@id && $id[0] && exists $sheet->{$id[0]}) {
+	my ($c, $r) = $sheet->cell2cr ($id[0]);
+	return $sheet->{attr}[$c][$r];
+	}
+    undef;
+    } # attr
+
 sub maxrow {
     my $sheet = shift;
     return $sheet->{maxrow};
@@ -989,16 +1005,32 @@ sub maxcol {
     return $sheet->{maxcol};
     } # maxrow
 
+sub col2label {
+    $_[0] =~ m/::/ and shift; # class unused
+    return Spreadsheet::Read::col2label (@_);
+    } # col2label
+
 sub cr2cell {
-    my $class = shift;
+    $_[0] =~ m/::/ and shift; # class unused
     return Spreadsheet::Read::cr2cell (@_);
     } # cr2cell
 
 sub cell2cr {
-    my $class = shift;
+    $_[0] =~ m/::/ and shift; # class unused
     return Spreadsheet::Read::cell2cr (@_);
     } # cell2cr
 
+sub label {
+    my ($sheet, $label) = @_;
+    defined $label and $sheet->{label} = $label;
+    return $sheet->{label};
+    } # label
+
+sub active {
+    my $sheet = shift;
+    return $sheet->{active};
+    } # label
+
 # my @row = $sheet->cellrow (1);
 sub cellrow {
     my ($sheet, $row) = @_;
@@ -1014,6 +1046,21 @@ sub row {
     map { $sheet->{$sheet->cr2cell ($_, $row)} } 1..$sheet->{maxcol};
     } # row
 
+# my @col = $sheet->cellcolumn (1);
+sub cellcolumn {
+    my ($sheet, $col) = @_;
+    defined $col && $col > 0 && $col <= $sheet->{maxcol} or return;
+    my $s = $sheet->{cell};
+    map { $s->[$col][$_] } 1..$sheet->{maxrow};
+    } # cellcolumn
+
+# my @col = $sheet->column (1);
+sub column {
+    my ($sheet, $col) = @_;
+    defined $col && $col > 0 && $col <= $sheet->{maxcol} or return;
+    map { $sheet->{$sheet->cr2cell ($col, $_)} } 1..$sheet->{maxrow};
+    } # column
+
 # Convert {cell}'s [column][row] to a [row][column] list
 # my @rows = $sheet->rows ();
 sub rows {
@@ -1077,6 +1124,7 @@ The data is returned as an array reference:
           ],
         attr    => [],
         merged  => [],
+        active  => 1,
         A1      => 1,
         B5      => "Nugget",
         },
@@ -1089,7 +1137,8 @@ The data is returned as an array reference:
 To keep as close contact to spreadsheet users, row and column 1 have
 index 1 too in the C<cell> element of the sheet hash, so cell "A1" is
 the same as C<cell> [1, 1] (column first). To switch between the two,
-there are two helper functions available: C<cell2cr ()> and C<cr2cell ()>.
+there are helper functions available: C<cell2cr ()>, C<cr2cell ()>,
+and C<col2label ()>.
 
 The C<cell> hash entry contains unformatted data, while the hash entries
 with the traditional labels contain the formatted values (if applicable).
@@ -1175,11 +1224,9 @@ See L</Cell Attributes> below.
 =item clip
 
 If set, L<C<ReadData>|/ReadData> will remove all trailing rows and columns
-per sheet that have no visual data. If a sheet has no data at all, the
-sheet will be skipped entirely when this attribute is true.
-
-This option is only valid if L<C<cells>|/cells> is true. The default value
-is true if L<C<cells>|/cells> is true, and false otherwise.
+per sheet that have no data, where no data means only undefined or empty
+cells (after optional stripping). If a sheet has no data at all, the sheet
+will be skipped entirely when this attribute is true.
 
 =item strip
 
@@ -1193,6 +1240,24 @@ leading-whitespace from every field.
     2      n/a     strip
     3     strip    strip
 
+=item pivot
+
+Swap all rows and columns.
+
+When a sheet contains data like
+
+  A1  B1  C1      E1
+  A2      C2  D2
+  A3  B3  C3  D3  E3
+
+using C<pivot> will return the sheet data as
+
+  A1  A2  A3
+  B1      B3
+  C1  C2  C3
+      D2  D3
+  E1      E3
+
 =item sep
 
 Set separator for CSV. Default is comma C<,>.
@@ -1222,11 +1287,29 @@ The value determines how much diagnostics are dumped (using
 L<Data::Peek|https://metacpan.org/release/Data-Peek>).  A value of C<9>
 and higher will dump the entire structure from the back-end parser.
 
+=item passwd
+
+Use this password to decrypt password protected spreadsheet.
+
+Currently only supports Excel.
+
 =back
 
 All other attributes/options will be passed to the underlying parser if
 that parser supports attributes.
 
+=head3 col2label
+
+ my $col_id = col2label (col);
+
+ my $col_id = $book->col2label (col);  # OO
+
+C<col2label ()> converts a C<(column)> (1 based) to the letters used in the
+traditional cell notation:
+
+  my $id = col2label ( 4); # $id now "D"
+  my $id = col2label (28); # $id now "AB"
+
 =head3 cr2cell
 
  my $cell = cr2cell (col, row);
@@ -1347,6 +1430,10 @@ Return the numbered or named sheet out of the book. Will return C<undef> if
 there is no match. Will not work for sheets I<named> with a number between 1
 and the number of sheets in the book.
 
+With named sheets will first try to use the list of sheet-labels as stored in
+the control structure. If no match is found, it will scan the actual labels
+of the sheets. In that case, it will return the first matching sheet.
+
 If defined, the returned sheet will be of class C<Spreadsheet::Read::Sheet>.
 
 =head3 add
@@ -1363,13 +1450,13 @@ If defined, the returned sheet will be of class C<Spreadsheet::Read::Sheet>.
 
  my $col = $sheet->maxcol;
 
-Return the index of the last in-use column in the sheet.
+Return the index of the last in-use column in the sheet. This index is 1-based.
 
 =head3 maxrow
 
  my $row = $sheet->maxrow;
 
-Return the index of the last in-use row in the sheet.
+Return the index of the last in-use row in the sheet. This index is 1-based.
 
 =head3 cell
 
@@ -1379,6 +1466,24 @@ Return the index of the last in-use row in the sheet.
 Return the value for a cell. Using tags will return the formatted value,
 using column and row will return unformatted value.
 
+=head3 attr
+
+ my $cell = $sheet->attr ("A3");
+ my $cell = $sheet->attr (1, 3);
+
+Return the attributes of a cell. Only valid if attributes are enabled through
+option C<attr>.
+
+=head3 col2label
+
+ my $col_id = $sheet->col2label (col);
+
+C<col2label ()> converts a C<(column)> (1 based) to the letters used in the
+traditional cell notation:
+
+  my $id = $sheet->col2label ( 4); # $id now "D"
+  my $id = $sheet->col2label (28); # $id now "AB"
+
 =head3 cr2cell
 
  my $cell = $sheet->cr2cell (col, row);
@@ -1399,6 +1504,22 @@ pair (1 based):
   my ($col, $row) = $sheet->cell2cr ("D14"); # returns ( 4, 14)
   my ($col, $row) = $sheet->cell2cr ("AB4"); # returns (28,  4)
 
+=head3 col
+
+ my @col = $sheet->column ($col);
+
+Get full column of formatted values (like C<< $sheet->{C1} .. $sheet->{C9} >>)
+
+Note that the indexes in the returned list are 0-based.
+
+=head3 cellcolumn
+
+ my @col = $sheet->cellcolumn ($col);
+
+Get full column of unformatted values (like C<< $sheet->{cell}[3][1] .. $sheet->{cell}[3][9] >>)
+
+Note that the indexes in the returned list are 0-based.
+
 =head3 row
 
  my @row = $sheet->row ($row);
@@ -1424,6 +1545,23 @@ Convert C<{cell}>'s C<[column][row]> to a C<[row][column]> list.
 Note that the indexes in the returned list are 0-based, where the
 index in the C<{cell}> entry is 1-based.
 
+=head3 label
+
+ my $label = $sheet->label;
+ $sheet->label ("New sheet label");
+
+Set a new label to a sheet. Note that the index in the control structure will
+I<NOT> be updated.
+
+=head3 active
+
+ my $sheet_is_active = $sheet->active;
+
+Returns 1 if the selected sheet is active, otherwise returns 0.
+
+Currently only works on XLS (as of Spreadsheed::ParseExcel-0.61).
+CSV is always active.
+
 =head2 Using CSV
 
 In case of CSV parsing, L<C<ReadData>|/ReadData> will use the first line of
@@ -1445,6 +1583,11 @@ See L<Text::CSV_XS|https://metacpan.org/pod/Text-CSV_XS> for documentation.
  my $ss = ReadData ("bad.csv");
  $ss->[0]{error} and say $ss->[0]{error}[1];
 
+As CSV has no sheet labels, the default label for a CSV sheet is its filename.
+For CSV, this can be overruled using the I<label> attribute:
+
+ my $ss = Spreadsheet::Read->new ("/some/place/test.csv", label => "Test");
+
 =head2 Cell Attributes
 X<merged>
 
@@ -1507,6 +1650,8 @@ If a cell itself is not hidden, but the parser holds the information that
 either the row or the column (or both) the field is in is hidden, the flag
 is inherited into the cell attributes.
 
+The entries C<maxrow> and C<maxcol> are 1-based.
+
 =head3 Merged cells
 
 Note that only
@@ -1732,6 +1877,26 @@ names C<meta>, or just be new values in the C<attr> hashes.
 
 I consider adding any spreadsheet interface that offers a usable API.
 
+Under investigation:
+
+=over 2
+
+=item Gnumeric (.gnumeric)
+
+I have seen no existing CPAN module yet.
+
+It is gzip'ed XML
+
+=item Kspread (.ksp)
+
+Now knows as Calligra Sheets.
+
+I have seen no existing CPAN module yet.
+
+It is XML in ZIP
+
+=back
+
 =item Alternative parsers for existing formats
 
 As long as the alternative has a good reason for its existence, and the
@@ -1820,7 +1985,7 @@ H.Merijn Brand, <h.m.brand@xs4all.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2016 H.Merijn Brand
+Copyright (C) 2005-2018 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
